#!/bin/bash
# This script works with a tar archvie
# called upgrade.bin in either /var/volatile
# or on the SD card.
#
# This is what the tar file upgrade file
# should look like to create an ipk upgrade
# file.  IPK files should be in all,
# arm926ejste, and mtcdt (for mtcdt)

# Blinking stops when we get to the install script.
# LED D is lit while waiting for radio-reset
# to complete.
# LED C is lit when radio-reset is complete.
# Lights start to blink when we start
# to install and remove packages.

flash_root=/
umount_all() {
    echo "Deactivating swap..."
    swapoff -a
    # We leave /proc mounted.
    echo "Unmounting local filesystems..."
    grep -q /mnt/ram /proc/mounts && mount -o remount,ro /mnt/ram
    umount -f -a -r > /dev/null 2>&1   mount -o remount,ro / }
}

function cleanup {
    sync
    sleep 2
    mount -o remount,ro ${flash_root}
    # flash_root is not going to be umounted   sed -i -e "\\|${flash_root}| d" /etc/mtab
    umount_all
    exit 1
}

trap cleanup exit

date
echo "Starting install.sh for 5.x.y to 5.1.0 upgrade" | tee /var/config/upgrade.log

# additional optional work.
(
  cd flash-upgrade
  if [[ -x ./extra.sh ]] && md5sum -c extra.sh.md5 ; then
    ./extra.sh
  fi
)

KREV=$(uname -r | sed 's/[-A-Za-z].*//')
OIFS="${IFS}"
IFS='.'
set $KREV
IFS="${OIFS}"

if ! ((($1 == 4)) && (($2 == 9)) && (($3 == 87))) ; then
    echo "Your kernel is $KREV  and must be version 4.9.87" | tee -a /var/config/upgrade.log
    echo "You must be on mLinux 5.0.0 first" | tee -a /var/config/upgrade.log
    exit 1
fi

if opkg list angel || opkg list wifiup || opkg list mts-xavier ; then
    echo "This install is not for AEP"  | tee -a /var/config/upgrade.log
    echo "You must use an AEP compatible upgrade"  | tee -a /var/config/upgrade.log
	exit 1
fi

dmesg >/var/config/upgrade.log
echo "Modules installed:" | tee -a /var/config/upgrade.log
/bin/lsmod  | tee -a /var/config/upgrade.log
t0=$(sed 's/\..*//' /proc/uptime)

NAME=install.sh

echo "$NAME starting in directory $(pwd)"
# Need more room
/bin/df -h
cd flash-upgrade/update-5.1.8-5.2.7
if ! md5sum -c md5sum.txt ; then
	echo Upgrade file is corrupted
	exit 1
fi

if [[ -d repo/mtcdt ]] ; then
	if ! [[ -f $(ls repo/mtcdt/kernel-module-rs9113_*) ]] ; then
		if [[ -f /opt/rs9113/onebox_gpl.ko ]] ; then
			date=$(date)
			echo "$date: Device has rs9113 installed, and no rs9113 package in upgrade" | tee -a /var/config/upgrade.log
			echo 'Contact Multitech Support for rs9113 upgrade package'  | tee -a /var/config/upgrade.log
			exit 1
		fi
	fi
fi

id=$(id -u)
if ((id != 0)) ; then
   echo 'You must be root (admin in AEP) to install packages.'
   exit 1
fi

localcnf=local.conf

if ! [[ -f ${localcnf} ]] ; then
    echo "Cannot find ${localcnf}"
    echo "This file must be in the current directory."
    exit 1
fi

dir=$(pwd)

sed -i "s?FLASHDIR?${dir}?" ${localcnf}

cat ${localcnf}

rm /var/volatile/flash-upgrade/upgrade.bin

# Copy commands for busybox upgrade:
mkdir /var/volatile/busybox
cp /bin/busybox* /var/volatile/busybox
ls -l /var/volatile/busybox
for d in /bin /sbin /usr/bin /usr/sbin ; do
    pushd $d
    echo searching $(pwd) for links
    for l in $(find . -type l) ; do
	target=$(readlink -f $l)
	base=$(basename $target)
	if [[ $base =~ busybox ]] ; then
            echo "ln -s /var/volatile/busybox/$base /var/volatile/busybox/$l"
	    ln -s /var/volatile/busybox/$base /var/volatile/busybox/$l
	fi
    done
    popd
done
PATH=/var/volatile/busybox:${PATH}
echo 'Where is sed?'
type sed
sed --version
echo "PATH is $PATH"

cat ${localcnf}

pkglist_file="install_list.txt"

rmv_file="remove_list.txt"
echo 'Remove obsolete packages'
if [[ -r ${rmv_file} ]] ; then
    OIFS="${IFS}"
    IFS=$'\n'
    for p in $(/bin/cat ${rmv_file}) ; do
        echo /usr/bin/opkg  --force-remove --force-depends --force-downgrade --force-reinstall -t /var/volatile/ -f "${localcnf}" remove $p
        /usr/bin/opkg --force-remove --force-depends --force-downgrade --force-reinstall -t /var/volatile/ -f "${localcnf}" remove $p
        sync
    done
    IFS="${OIFS}"
fi
sync;sleep 1;sync

echo "Update package list ... /usr/bin/opkg -f ${localcnf} update"   | tee -a /var/config/upgrade.log
/usr/bin/opkg -t /var/volatile/ -f "${localcnf}" update   | tee -a /var/config/upgrade.log

echo 'Install new packages'
if [[ -r ${pkglist_file} ]] ; then
    OIFS="${IFS}"
    IFS=$'\n'
    for p in $(/bin/cat ${pkglist_file}) ; do
        echo /usr/bin/opkg  --force-remove --force-depends --force-downgrade --force-reinstall -t /var/volatile/ -f "${localcnf}" install $p
        /usr/bin/opkg --force-remove --force-depends --force-downgrade --force-reinstall -t /var/volatile/ -f "${localcnf}" install $p
        sync
    done
    IFS="${OIFS}"
fi


sync;sleep 1;sync
/usr/bin/opkg --force-remove --force-depends -t /var/volatile/ -f "${localcnf}" upgrade   | tee -a /var/config/upgrade.log
sync;sleep 1;sync

echo "Update /etc/mlinux-version"
version=$(/bin/grep mLinux /etc/mlinux-version | /bin/sed 's/mLinux //')

/bin/sed -ri "s/mLinux [0-9][^ ]* /mLinux ${version} /" /etc/issue
/bin/sed -ri "s/Version: [0-9][^ ]*/Version: ${version} /" /etc/issue

# /etc/volatile.cache is a cache of how to create /run and other
# volatile directories and files.
rm -f /etc/volatile.cache
echo 'Completed upgrade -- check space'
/bin/df -h


sync;sleep 1;sync

/bin/date
t1=$(sed 's/\..*//' /proc/uptime)
((minutes=(t1-t0)/60))
sync;sleep 1;sync
echo "Upgrade completed in $minutes minutes"  | tee -a /var/config/upgrade.log
cleanup
